/**
 * @file clause.h
 * @author Léo Perrin
 * @date Time-stamp: <2012-06-16 17:13:04 leo>
 */
#ifndef _CLAUSE_H_
#define _CLAUSE_H_

#include <vector>
#include <string>

namespace cnf {


/**
 * @class Clause
 * @brief This class is used to manipulate CNF clauses.
 * 
 * A CNF clause consist of the disjunction of an arbitrary number of
 * literals. This class makes it easier to use these in coordination
 * with the <a href="http://minisat.se/MiniSat.html">minisat2</a>
 * program.
 */
class Clause
{
	private:
		/**
		 * @brief The codes of the variables of the clause.
		 *
		 * If the literal is negated, it contains the opposite of its
		 * code instead of its code according to the DIMACS encoding.
		 */
		std::vector<int> literals;

	public:
		/**
		 * @brief Builds an empty Clause instance.
		 **/
		Clause();

		/**
		 * @brief Adds the code of the literal given as an
		 * argument to the Clause.
		 */
		void addLit(int);

		/**
		 * @brief Puts all the literals in a one line std::string.
		 *
		 * The aim of this method is to write a line of a DIMACS
		 * formatted file corresponding to this disjunction.
		 **/
		std::string toString();

                /**
                 * @brief Clears the content of the clause by clearing
                 * the literals attribute.
                 */
                void clear();
};


} // closing namespace

#endif
