/**
 * @file formula.cpp
 * @author Léo Perrin
 * @date Time-stamp: <2012-06-16 17:14:15 leo>
 */

#include <vector>
#include <iostream>
#include <fstream>
#include <stdlib.h>

#include "formula.h"

using namespace cnf;


Formula::Formula(VariableSet * S,std::ostream * out,
                 std::string comment)
{
	varSet = S;
        // initializing output
        output = out;
	(*output) << "c " << comment << std::endl
               << "p cnf " << varSet->getCard()
               << " 1 " << std::endl;
}


Formula::~Formula()
{
        flushCNF();
}


void Formula::newClause()
{
        bufferClause.clear();
}


void Formula::addLit(unsigned int s, bool sign, unsigned int i, unsigned int j)
{
	if (sign)
		bufferClause.addLit((int)varSet->getVarCode(s,i,j));
	else
		bufferClause.addLit((-1)*(int)varSet->getVarCode(s,i,j));
}


void Formula::addLit(unsigned int s, bool sign, unsigned int i, unsigned int j, unsigned int k)
{
	if (sign)
		bufferClause.addLit((int)varSet->getVarCode(s,i,j,k));
	else
		bufferClause.addLit((-1)*(int)varSet->getVarCode(s,i,j,k));
}


void Formula::addLitCode(int c)
{
        if (
                ((c < 0)  &&  ((-1)*c > (int)varSet->getCard()))
                || ((c >= 0)  &&  (c > (int)varSet->getCard()))
                )
        {
                std::cout<<"ERROR in Formula::addLitCode(), the code is"
                        " too big: c="<<c<<" is greater than the varSet"
                        " size, "<<varSet->getCard()<<std::endl;
                exit(1);
        }
        else
                bufferClause.addLit(c);
}


void Formula::pushClause()
{
	cnfs.push_back(bufferClause);
        if (cnfs.size() > CLAUSE_BUFFER_SIZE)
                flushCNF();
}


void Formula::flushCNF()
{
        for (unsigned int i=0; i<cnfs.size(); i++)
                (*output) << cnfs[i].toString() << "\n";
        output->flush();
        cnfs.clear();
}
